#
#	MacUndifference.make
#
# Macintosh MPW Makefile for the Eclectus Undifference utility
# Copyright (C) 1992-96 Eclectus (D. John Anderson, Alan B. Harper).

# This file is part of the Eclectus integration utilities.

# Eclectus integration utilities are free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 1, or
# (at your option) any later version.

# Eclectus integration utilities are distributed in the hope that they
# will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the Eclectus integration utilities; see the file COPYING.
# If not, write to the Free Software Foundation, 675 Mass Ave, Cambridge,
# MA 02139, USA.

# This make file has the following targets
#
#		Clean:						Remove all derived files, create directories necessary for other targets below.
#
#		All/<no target>:	Undifference and UndifferenceDebug as fat tools*
#		Release:					Fat Undifference
#		PPC:							PPC only Undifference
#		68K:							68K only Undifference
#		Debug							Fat UndifferenceDebug*
#		DebugPPC:					PPC only UndifferenceDebug*
#		Debug68K:					68K only UndifferenceDebug*
#
#		Install:					Install fat Undifference and UndifferenceDebug*
#		InstallRelease:		Install fat Undifference*
#		InstallPPC:				Install PPC Undifference*
#		Install68K:				Install 68K Undifference*
#		InstallDebug:			Install fat UndifferenceDebug*
#		InstallDebugPPC:	Install PPC only UndifferenceDebug*
#		InstallDebug68K:	Install 68K only UndifferenceDebug*
#
#	*	The code for UndifferenceDebug is not included in the standard distribution, since it is used
#		only to test the code. Write Alan Harper (aharper@dnai.com) if you have a need for this code.
#		Also, the Install... targets place the tools in {MPW}ECTools, which may not be where you
#		want them. Change BIN_DIR to adjust this.

MWCPPC_OPTIONS = 
	-d DF_MACHINE_MACINTOSH 
	-d DF_COMPILER_MW 
	-d DF_CHIP_PPC 
	-mapcr 
	-maxerrors 10 
	-once 
	-opt all 
	-proto strict 
	-r 
	-str readonly 
	-trigraphs on 
	-w full


MWC68K_OPTIONS = 
	-d DF_MACHINE_MACINTOSH 
	-d DF_COMPILER_MW 
	-d DF_CHIP_68K 
	-enum int 
	-mapcr 
	-maxerrors 10 
	-mbg off 
	-mc68020 
	-model codesmart 
	-once 
	-opt all 
	-proto strict 
	-r 
	-str pool 
	-sym off 
	-trigraphs on 
	-w full

ADDL_MWCPPC_OPTIONS = 

ADDL_MWC68K_OPTIONS = 

MWLINKPPC_OPTIONS = 
	-d 
	-c 'MPS ' 
	-t 'MPST'

MWLINK68K_OPTIONS = -mpwtool

MWLINKPPC_LIBS = 
	"{MWPPCLibraries}"MWMPWCRuntime.Lib 
	"{MWPPCLibraries}"'MPW ANSI.C.PPC.Lib' 
	"{MWPPCLibraries}"PPCToolLibs.o 
	"{SharedLibraries}"InterfaceLib 
	"{SharedLibraries}"MathLib 

MWLINK68K_LIBS = 
	"{MW68KLibraries}"MPWRuntime.68K.lib 
	"{MW68KLibraries}"'MPW ANSI (4i/8d) C.68K.Lib' 
	"{MW68KLibraries}"'MathLib68K (4i/8d).Lib' 
	"{MW68KLibraries}"ToolLibs.o 
	"{MW68KLibraries}"MacOS.Lib 

DERIVED_DIR = :UndifferenceR.i

DERIVED_DEBUG_DIR = :UndifferenceD.i

TOOL_NAME = Undifference

DEBUG_TOOL_NAME = UndifferenceDebug

INSTALL_AS_NAME = Undifference

INSTALL_FROM_NAME = Undifference

BIN_DIR = {MPW}ECTools

LOCAL_BIN_DIR = :MacTools

UNDIFFERENCE_TO_OBERISH_DIR = ::Oberish

H_FILES = 
	diff.h

UNDIFFERENCE_PPC_OBJS = 
	{DERIVED_DIR}:io.c.x 
	{DERIVED_DIR}:util.c.x 
	{DERIVED_DIR}:Undifference.c.x 
	{DEBUG_PPC_OBJS}

UNDIFFERENCE_68K_OBJS = 
	{DERIVED_DIR}:io.c.o 
	{DERIVED_DIR}:util.c.o 
	{DERIVED_DIR}:Undifference.c.o 
	{DEBUG_68K_OBJS}

DEBUG_PPC_OBJS =

DEBUG_68K_OBJS =

Release  Undifference

All  Release Debug

PPC  Undifference.PPC

68K  Undifference.68K

Debug  UndifferenceDebug

DebugPPC  UndifferenceDebug.PPC

Debug68K  UndifferenceDebug.68K

Undifference  {DERIVED_DIR}:Undifference

Undifference.PPC  {DERIVED_DIR}:Undifference.PPC

Undifference.68K  {DERIVED_DIR}:Undifference.68K

UndifferenceDebug  
	Make -f MacUndifference.make 
		-d DEBUG_PPC_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.x" 
		-d DEBUG_68K_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.o" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d ADDL_MWCPPC_OPTIONS="-d DF_DEBUG -i {UNDIFFERENCE_TO_OBERISH_DIR}" 
		-d ADDL_MWC68K_OPTIONS="-d DF_DEBUG -i {UNDIFFERENCE_TO_OBERISH_DIR}" 
		-d TOOL_NAME={DEBUG_TOOL_NAME} 
		{DERIVED_DEBUG_DIR}:{DEBUG_TOOL_NAME}

UndifferenceDebug.PPC 
	Make -f MacUndifference.make 
		-d DEBUG_PPC_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.x" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d ADDL_MWCPPC_OPTIONS="-d DF_DEBUG -i {UNDIFFERENCE_TO_OBERISH_DIR}" 
		-d TOOL_NAME={DEBUG_TOOL_NAME} 
		{DERIVED_DEBUG_DIR}:{DEBUG_TOOL_NAME}.PPC

UndifferenceDebug.68K 
	Make -f MacUndifference.make 
		-d DEBUG_68K_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.o" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d ADDL_MWC68K_OPTIONS="-d DF_DEBUG -i {UNDIFFERENCE_TO_OBERISH_DIR}" 
		-d TOOL_NAME={DEBUG_TOOL_NAME} 
		{DERIVED_DEBUG_DIR}:{DEBUG_TOOL_NAME}.68K

{UNDIFFERENCE_PPC_OBJS}  {H_FILES}

{UNDIFFERENCE_68K_OBJS}  {H_FILES}

{DERIVED_DIR}:  :

{DERIVED_DIR}:  {UNDIFFERENCE_TO_OBERISH_DIR}:

{DERIVED_DIR}:{TOOL_NAME}  {DERIVED_DIR}:{TOOL_NAME}.PPC {DERIVED_DIR}:{TOOL_NAME}.68K
	Duplicate -y {DERIVED_DIR}:{TOOL_NAME}.PPC {DERIVED_DIR}:{TOOL_NAME}
	Rez -append -o {DERIVED_DIR}:{TOOL_NAME} -d RSRCNAME=""{DERIVED_DIR}:{TOOL_NAME}.68K"" MacCopyRSRC.r

{DERIVED_DIR}:{TOOL_NAME}.PPC   {UNDIFFERENCE_PPC_OBJS}
	MWLinkPPC {MWLINKPPC_LIBS} {UNDIFFERENCE_PPC_OBJS} {MWLINKPPC_OPTIONS} -o {Targ}
	Rez -d APPNAME="{TOOL_NAME}" MacGenericCFRG.r -o {Targ}

{DERIVED_DIR}:{TOOL_NAME}.68K   {UNDIFFERENCE_68K_OBJS}
	MWLink68K {MWLINK68K_LIBS} {UNDIFFERENCE_68K_OBJS} {MWLINK68K_OPTIONS} -o {Targ}

Install  InstallRelease InstallDebug

InstallRelease  {DERIVED_DIR}:{INSTALL_FROM_NAME}
	If not `Exists {BIN_DIR}:`
		NewFolder {BIN_DIR}:
	End
	Duplicate -y {DERIVED_DIR}:{INSTALL_FROM_NAME} {BIN_DIR}:{INSTALL_AS_NAME}
	If not `Exists {LOCAL_BIN_DIR}:`
		NewFolder {LOCAL_BIN_DIR}:
	End
	Duplicate -y {DERIVED_DIR}:{INSTALL_FROM_NAME} {LOCAL_BIN_DIR}:{INSTALL_AS_NAME}.mpw

InstallPPC 
	Make -f MacUndifference.make InstallRelease 
		-d INSTALL_FROM_NAME="Undifference.PPC" 
		-d TOOL_NAME="Undifference"

Install68K 
	Make -f MacUndifference.make InstallRelease 
		-d INSTALL_FROM_NAME="Undifference.68K" 
		-d TOOL_NAME="Undifference"

InstallDebug 
	Make -f MacUndifference.make InstallRelease 
		-d ADDL_MWCPPC_OPTIONS="-d DF_DEBUG -i {UNDIFFERENCE_TO_OBERISH_DIR}" 
		-d ADDL_MWC68K_OPTIONS="-d DF_DEBUG -i {UNDIFFERENCE_TO_OBERISH_DIR}" 
		-d DEBUG_PPC_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.x" 
		-d DEBUG_68K_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.o" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d INSTALL_FROM_NAME="UndifferenceDebug" 
		-d INSTALL_AS_NAME="UndifferenceDebug" 
		-d TOOL_NAME="UndifferenceDebug"

InstallDebugPPC 
	Make -f MacUndifference.make InstallRelease 
		-d ADDL_MWCPPC_OPTIONS="-d DF_DEBUG -i {UNDIFFERENCE_TO_OBERISH_DIR}" 
		-d DEBUG_PPC_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.x" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d INSTALL_FROM_NAME="UndifferenceDebug.PPC" 
		-d INSTALL_AS_NAME="UndifferenceDebug" 
		-d TOOL_NAME="UndifferenceDebug"

InstallDebug68K 
	Make -f MacUndifference.make InstallRelease 
		-d ADDL_MWC68K_OPTIONS="-d DF_DEBUG -i {UNDIFFERENCE_TO_OBERISH_DIR}" 
		-d DEBUG_68K_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.o" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d INSTALL_FROM_NAME="UndifferenceDebug.68K" 
		-d INSTALL_AS_NAME="UndifferenceDebug" 
		-d TOOL_NAME="UndifferenceDebug"

Clean 
	Delete -y -i {DERIVED_DIR}
	NewFolder {DERIVED_DIR}
	Delete -y -i {DERIVED_DEBUG_DIR}
	NewFolder {DERIVED_DEBUG_DIR}

.c.x  .c
	MWCPPC {MWCPPC_OPTIONS} {ADDL_MWCPPC_OPTIONS} {DepDir}{Default}.c -o {Targ}

.c.o  .c
	MWC68K {MWC68K_OPTIONS} {ADDL_MWC68K_OPTIONS} {DepDir}{Default}.c -o {Targ}
